mod button;
pub use button::ButtonSet;

#[derive(Clone, Copy, Debug)]
#[repr(u8)]
pub enum MouseButton {
    Left,
    Right,
    Middle,
    ScrollUp,
    ScrollDown,
    ExtraButton1,
    ExtraButton2,
    ExtraButton3,
    ExtraButton4,
}

impl Into<u8> for MouseButton {
    fn into(self) -> u8 {
        self as u8
    }
}

pub type MouseButtonSet = ButtonSet<MouseButton>;

#[derive(Default)]
pub struct MouseState {
    pub pos: kahlo::math::PixelPoint,
    pub last_pos: kahlo::math::PixelPoint,
    pub buttons: ButtonSet<MouseButton>,
    pub last_buttons: ButtonSet<MouseButton>,
}

impl MouseState {
    pub fn pressed(&self) -> MouseButtonSet {
        self.buttons & !self.last_buttons
    }
    pub fn released(&self) -> MouseButtonSet {
        !self.buttons & self.last_buttons
    }
}

#[derive(Default)]
pub struct InputState {
    pub mouse: MouseState,
}

impl InputState {
    /// Note the passing of a frame.
    pub fn tick(&mut self) {
        self.mouse.last_pos = self.mouse.pos;
        self.mouse.last_buttons = self.mouse.buttons;
    }
}
