use crate::{
    component::Component,
    input::InputState,
    layout::{LayoutNode, LayoutNodeAccess, LayoutNodeContainer},
    theme::Theme,
    ui::UIHandle,
    widget::Widget, window::RenderFormat,
};

pub struct PlainGroup<C: Component> {
    lnode: LayoutNode,
    children: Vec<Box<dyn Widget<C>>>,
}

impl<C: Component> PlainGroup<C> {
    pub fn new(uih: &UIHandle) -> Self {
        Self {
            lnode: uih.new_layout_node(),
            children: vec![],
        }
    }

    pub fn append(&mut self, widget: Box<dyn Widget<C>>) {
        self.children.push(widget);
    }
}

impl<C: Component> LayoutNodeContainer for PlainGroup<C> {
    fn layout_node(&self) -> &LayoutNode {
        &self.lnode
    }

    fn layout_child(&self, ndx: usize) -> Option<LayoutNodeAccess> {
        self.children.get(ndx).map(|w| w.layout_node())
    }

    fn layout_child_count(&self) -> usize {
        self.children.len()
    }
}

impl<C: Component> Widget<C> for PlainGroup<C> {
    fn layout_node(&self) -> LayoutNodeAccess {
        LayoutNodeAccess::new(self)
    }
    fn layout_node_mut(&mut self) -> &mut LayoutNode {
        &mut self.lnode
    }
    fn poll(&mut self, uih: &mut UIHandle, input_state: Option<&InputState>) -> Vec<C::Msg> {
        self.children
            .iter_mut()
            .map(|w| w.poll(uih, input_state))
            .flatten()
            .collect()
    }
    fn render(&self, theme: &Theme, target: &mut kahlo::BitmapMut<RenderFormat>) {
        for child in self.children.iter() {
            child.render(theme, target);
        }
    }
}
